package de.tud.kom.p2psim.impl.network.fairshare;

/**
 * A Connection connects two distinct Nodes. It is directed, that is, it
 * connects a source to a destination. Connections are Comparable in that a
 * Connection with a smaller "bid" is smaller than a Connection with a bigger
 * "bid" (see {@link #getMinBid()}).
 * 
 * @author Tobias Lauinger
 */
public interface Connection extends Comparable<Connection> {

	/**
	 * Returns one of the end points of this Connection.
	 * 
	 * @param destination
	 *            true if the sink is to be returned (else the source).
	 * 
	 * @return the download/upload Node of this Connection.
	 */
	public Node getNode(boolean destination);

	/**
	 * @return true if this Connection is currently used (false if idle).
	 */
	public boolean isInUse();

	/**
	 * Resets the state of this Connection for a new, clean run of the fair
	 * share algorithm. The state that will be cleared is the assigned
	 * bandwidth.
	 */
	public void reset();

	/**
	 * @return the bandwidth that has been assigned to this Connection by the
	 *         fair share algorithm. Returns 0 if the bandwidth has not yet been
	 *         determined.
	 */
	public double getBandwidth();

	/**
	 * @return the minimum bandwidth bid of the two connected Nodes.
	 */
	public double getMinBid();

	/**
	 * Assigns the given bandwidth to this Connection and updates the source and
	 * destination Nodes of this Connection.
	 * 
	 * @param bandwidth
	 *            the bandwidth of this Connection.
	 */
	public void assignBandwidth(double bandwidth);

}
