package de.tud.kom.p2psim.impl.network.fairshare.position;

import de.tud.kom.p2psim.api.common.Position;
import de.tudarmstadt.maki.simonstrator.api.Randoms;

/**
 * This class implements a NetPosition for a GNP-Based calculation of round trip
 * times. Therefore it includes methods for error estimation and methods for
 * positioning by a downhill simplex algorithm in the GnpSpace class
 * 
 * @author Gerald Klunker
 * @version 0.1, 09.01.2008
 * 
 */
public class FairshareGnpPosition implements Position {

	private static final long serialVersionUID = -1103996725403557900L;

	private final double[] gnpCoordinates;

	private double error = -1.0;

	/**
	 * 
	 * @param gnpCoordinates
	 *            coordinate array for new position
	 */
	public FairshareGnpPosition(double[] gnpCoordinates) {
		super();
		this.gnpCoordinates = gnpCoordinates;
	}

	/**
	 * 
	 * @param dimension
	 * @param maxDiversity
	 */
	public void diversify(double[][] dimension, double maxDiversity) {
		for (int c = 0; c < this.gnpCoordinates.length; c++) {
			double rand = (2 * maxDiversity * Randoms.getRandom(
					FairshareGnpPosition.class)
					.nextDouble()) - maxDiversity;
			gnpCoordinates[c] = gnpCoordinates[c] + (rand * dimension[c][2]);
		}
		error = -1.0;
	}

	/**
	 * reposition
	 * 
	 * @param pos
	 *            position in the coordinate array
	 * @param value
	 *            new value at position pos
	 */
	public void setGnpCoordinates(int pos, double value) {
		gnpCoordinates[pos] = value;
		error = -1.0;
	}

	/**
	 * 
	 * @return number of dimensions
	 */
	public int getNoOfDimensions() {
		return gnpCoordinates.length;
	}

	/**
	 * 
	 * @param pos
	 *            position in the coordinate array
	 * @return value at position pos
	 */
	public double getGnpCoordinates(int pos) {
		return gnpCoordinates[pos];
	}

	/**
	 * 
	 * @return Comma-separated list of coordinates
	 */
	public String getCoordinateString() {
		if (gnpCoordinates.length == 0) {
			return "";
		} else {
			String result = String.valueOf(gnpCoordinates[0]);
			for (int c = 1; c < gnpCoordinates.length; c++)
				result = result + "," + gnpCoordinates[1];
			return result;
		}
	}

	@Override
	public double getDistance(Position netPosition) {
		FairshareGnpPosition coord = (FairshareGnpPosition) netPosition;
		double distance = 0.0;
		for (int c = 0; c < gnpCoordinates.length; c++)
			distance += Math.pow(
					gnpCoordinates[c] - coord.getGnpCoordinates(c), 2);
		return Math.sqrt(distance);
	}

	@Override
	public int getTransmissionSize() {
		// 2 * size(double)
		return 2 * 8;
	}

	@Override
	public double getAngle(Position target) {
		throw new AssertionError(
				"getAngle is not defined for this Position-Type");
	}

	@Override
	public Position getTarget(double distance, double angle) {
		throw new AssertionError(
				"getTarget is not defined for this Position-Type");
	}

	public double[] getCoords() {
		return gnpCoordinates;
	}

	@Override
	public FairshareGnpPosition clone() {
		return new FairshareGnpPosition(gnpCoordinates);
	}
}
